"""Tests module for scientific-spinbox.

This module aggregates tests for each part of scientific-spinbox,
like backend, validators and widget. Uses pytest as the test framework.

By running tests, we ensure that all core features and behaviour are
working as expected.

Authors:
    - Breno H. Pelegrin S. <breno.pelegrin@usp.br>
"""