# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'dlg_testspinbox_shortexample_gui.ui'
#
# Created by: qtpy UI code generator 5.15.11
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from qtpy import QtCore, QtGui, QtWidgets  # noqa: E402, F401


class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(400, 255)
        self.gridLayout = QtWidgets.QGridLayout(Dialog)
        self.gridLayout.setObjectName("gridLayout")
        self.groupBox = QtWidgets.QGroupBox(Dialog)
        self.groupBox.setObjectName("groupBox")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout.setObjectName("verticalLayout")
        self.valueLabel = QtWidgets.QLabel(self.groupBox)
        self.valueLabel.setObjectName("valueLabel")
        self.verticalLayout.addWidget(self.valueLabel)
        self.unitLabel = QtWidgets.QLabel(self.groupBox)
        self.unitLabel.setObjectName("unitLabel")
        self.verticalLayout.addWidget(self.unitLabel)
        self.baseValueLabel = QtWidgets.QLabel(self.groupBox)
        self.baseValueLabel.setObjectName("baseValueLabel")
        self.verticalLayout.addWidget(self.baseValueLabel)
        self.baseUnitLabel = QtWidgets.QLabel(self.groupBox)
        self.baseUnitLabel.setObjectName("baseUnitLabel")
        self.verticalLayout.addWidget(self.baseUnitLabel)
        self.gridLayout.addWidget(self.groupBox, 4, 0, 1, 1)
        self.label_8 = QtWidgets.QLabel(Dialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_8.sizePolicy().hasHeightForWidth())
        self.label_8.setSizePolicy(sizePolicy)
        self.label_8.setObjectName("label_8")
        self.gridLayout.addWidget(self.label_8, 0, 0, 1, 1)
        self.testSpinBox1 = QtWidgets.QWidget(Dialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.testSpinBox1.sizePolicy().hasHeightForWidth())
        self.testSpinBox1.setSizePolicy(sizePolicy)
        self.testSpinBox1.setMinimumSize(QtCore.QSize(150, 30))
        self.testSpinBox1.setMaximumSize(QtCore.QSize(250, 30))
        self.testSpinBox1.setObjectName("testSpinBox1")
        self.gridLayout.addWidget(self.testSpinBox1, 2, 0, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(20, 50, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        self.gridLayout.addItem(spacerItem, 3, 0, 1, 1)
        spacerItem1 = QtWidgets.QSpacerItem(20, 5, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        self.gridLayout.addItem(spacerItem1, 5, 0, 1, 1)

        self.retranslateUi(Dialog)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        _translate = QtCore.QCoreApplication.translate
        Dialog.setWindowTitle(_translate("Dialog", "Test ScientificSpinBox"))
        self.groupBox.setTitle(_translate("Dialog", "Properties of interest"))
        self.valueLabel.setText(_translate("Dialog", "value:"))
        self.unitLabel.setText(_translate("Dialog", "unit:"))
        self.baseValueLabel.setText(_translate("Dialog", "baseValue:"))
        self.baseUnitLabel.setText(_translate("Dialog", "baseUnit:"))
        self.label_8.setText(_translate("Dialog", "ScientificSpinBox widget (controls time)"))
