# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'dlg_testspinbox_gui.ui'
#
# Created by: qtpy UI code generator 5.15.11
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from qtpy import QtCore, QtGui, QtWidgets  # noqa: E402, F401


class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(1037, 637)
        self.verticalLayout = QtWidgets.QVBoxLayout(Dialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.groupBox_2 = QtWidgets.QGroupBox(Dialog)
        self.groupBox_2.setObjectName("groupBox_2")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.groupBox_2)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.testSpinBox1 = QtWidgets.QWidget(self.groupBox_2)
        self.testSpinBox1.setObjectName("testSpinBox1")
        self.verticalLayout_4.addWidget(self.testSpinBox1)
        self.reprPropertiesGroupBox = QtWidgets.QGroupBox(self.groupBox_2)
        self.reprPropertiesGroupBox.setObjectName("reprPropertiesGroupBox")
        self.gridLayout_5 = QtWidgets.QGridLayout(self.reprPropertiesGroupBox)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.reprQuantityLabel = QtWidgets.QLabel(self.reprPropertiesGroupBox)
        self.reprQuantityLabel.setObjectName("reprQuantityLabel")
        self.gridLayout_5.addWidget(self.reprQuantityLabel, 1, 0, 1, 1)
        self.reprValueLabel = QtWidgets.QLabel(self.reprPropertiesGroupBox)
        self.reprValueLabel.setObjectName("reprValueLabel")
        self.gridLayout_5.addWidget(self.reprValueLabel, 0, 0, 1, 1)
        self.reprUnitLabel = QtWidgets.QLabel(self.reprPropertiesGroupBox)
        self.reprUnitLabel.setObjectName("reprUnitLabel")
        self.gridLayout_5.addWidget(self.reprUnitLabel, 2, 0, 1, 1)
        self.verticalLayout_4.addWidget(self.reprPropertiesGroupBox)
        self.groupBox_5 = QtWidgets.QGroupBox(self.groupBox_2)
        self.groupBox_5.setObjectName("groupBox_5")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.groupBox_5)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.baseValueLabel = QtWidgets.QLabel(self.groupBox_5)
        self.baseValueLabel.setObjectName("baseValueLabel")
        self.verticalLayout_3.addWidget(self.baseValueLabel)
        self.baseQuantityLabel = QtWidgets.QLabel(self.groupBox_5)
        self.baseQuantityLabel.setObjectName("baseQuantityLabel")
        self.verticalLayout_3.addWidget(self.baseQuantityLabel)
        self.baseUnitLabel = QtWidgets.QLabel(self.groupBox_5)
        self.baseUnitLabel.setObjectName("baseUnitLabel")
        self.verticalLayout_3.addWidget(self.baseUnitLabel)
        self.verticalLayout_4.addWidget(self.groupBox_5)
        self.groupBox_6 = QtWidgets.QGroupBox(self.groupBox_2)
        self.groupBox_6.setObjectName("groupBox_6")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.groupBox_6)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.internalTextLabel = QtWidgets.QLabel(self.groupBox_6)
        self.internalTextLabel.setObjectName("internalTextLabel")
        self.verticalLayout_2.addWidget(self.internalTextLabel)
        self.decimalsLabel = QtWidgets.QLabel(self.groupBox_6)
        self.decimalsLabel.setObjectName("decimalsLabel")
        self.verticalLayout_2.addWidget(self.decimalsLabel)
        self.lastTextLabel = QtWidgets.QLabel(self.groupBox_6)
        self.lastTextLabel.setObjectName("lastTextLabel")
        self.verticalLayout_2.addWidget(self.lastTextLabel)
        self.verticalLayout_4.addWidget(self.groupBox_6)
        self.verticalLayout.addWidget(self.groupBox_2)
        spacerItem = QtWidgets.QSpacerItem(20, 10, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        self.verticalLayout.addItem(spacerItem)
        self.groupBox = QtWidgets.QGroupBox(Dialog)
        self.groupBox.setAutoFillBackground(False)
        self.groupBox.setFlat(False)
        self.groupBox.setCheckable(False)
        self.groupBox.setObjectName("groupBox")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.groupBox)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.groupBox_3 = QtWidgets.QGroupBox(self.groupBox)
        self.groupBox_3.setObjectName("groupBox_3")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.groupBox_3)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.tooltipCheckBox = QtWidgets.QCheckBox(self.groupBox_3)
        self.tooltipCheckBox.setChecked(True)
        self.tooltipCheckBox.setObjectName("tooltipCheckBox")
        self.gridLayout_2.addWidget(self.tooltipCheckBox, 1, 0, 1, 1)
        self.thousandSepCheckBox = QtWidgets.QCheckBox(self.groupBox_3)
        self.thousandSepCheckBox.setChecked(True)
        self.thousandSepCheckBox.setObjectName("thousandSepCheckBox")
        self.gridLayout_2.addWidget(self.thousandSepCheckBox, 0, 0, 1, 1)
        self.integerRoundCheckBox = QtWidgets.QCheckBox(self.groupBox_3)
        self.integerRoundCheckBox.setEnabled(True)
        self.integerRoundCheckBox.setCheckable(True)
        self.integerRoundCheckBox.setObjectName("integerRoundCheckBox")
        self.gridLayout_2.addWidget(self.integerRoundCheckBox, 2, 0, 1, 1)
        self.gridLayout_3.addWidget(self.groupBox_3, 0, 0, 1, 1)
        self.applyButton = QtWidgets.QPushButton(self.groupBox)
        self.applyButton.setAutoDefault(False)
        self.applyButton.setObjectName("applyButton")
        self.gridLayout_3.addWidget(self.applyButton, 1, 0, 1, 2)
        self.groupBox_4 = QtWidgets.QGroupBox(self.groupBox)
        self.groupBox_4.setObjectName("groupBox_4")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.groupBox_4)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.label = QtWidgets.QLabel(self.groupBox_4)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)
        self.baseUnitTextEdit = QtWidgets.QTextEdit(self.groupBox_4)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.baseUnitTextEdit.sizePolicy().hasHeightForWidth())
        self.baseUnitTextEdit.setSizePolicy(sizePolicy)
        self.baseUnitTextEdit.setMinimumSize(QtCore.QSize(0, 25))
        self.baseUnitTextEdit.setMaximumSize(QtCore.QSize(16777215, 25))
        self.baseUnitTextEdit.setObjectName("baseUnitTextEdit")
        self.gridLayout.addWidget(self.baseUnitTextEdit, 0, 1, 1, 1)
        self.label_2 = QtWidgets.QLabel(self.groupBox_4)
        self.label_2.setObjectName("label_2")
        self.gridLayout.addWidget(self.label_2, 1, 0, 1, 1)
        self.displayUnitTextEdit = QtWidgets.QTextEdit(self.groupBox_4)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.displayUnitTextEdit.sizePolicy().hasHeightForWidth())
        self.displayUnitTextEdit.setSizePolicy(sizePolicy)
        self.displayUnitTextEdit.setMinimumSize(QtCore.QSize(0, 25))
        self.displayUnitTextEdit.setMaximumSize(QtCore.QSize(16777215, 25))
        self.displayUnitTextEdit.setObjectName("displayUnitTextEdit")
        self.gridLayout.addWidget(self.displayUnitTextEdit, 1, 1, 1, 1)
        self.label_3 = QtWidgets.QLabel(self.groupBox_4)
        self.label_3.setObjectName("label_3")
        self.gridLayout.addWidget(self.label_3, 2, 0, 1, 1)
        self.defaultValueTextEdit = QtWidgets.QTextEdit(self.groupBox_4)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.defaultValueTextEdit.sizePolicy().hasHeightForWidth())
        self.defaultValueTextEdit.setSizePolicy(sizePolicy)
        self.defaultValueTextEdit.setMinimumSize(QtCore.QSize(0, 25))
        self.defaultValueTextEdit.setMaximumSize(QtCore.QSize(16777215, 25))
        self.defaultValueTextEdit.setObjectName("defaultValueTextEdit")
        self.gridLayout.addWidget(self.defaultValueTextEdit, 2, 1, 1, 1)
        self.horizontalLayout.addLayout(self.gridLayout)
        spacerItem1 = QtWidgets.QSpacerItem(90, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.gridLayout_4 = QtWidgets.QGridLayout()
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.label_4 = QtWidgets.QLabel(self.groupBox_4)
        self.label_4.setObjectName("label_4")
        self.gridLayout_4.addWidget(self.label_4, 0, 0, 1, 1)
        self.defaultDecimalsTextEdit = QtWidgets.QTextEdit(self.groupBox_4)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.defaultDecimalsTextEdit.sizePolicy().hasHeightForWidth())
        self.defaultDecimalsTextEdit.setSizePolicy(sizePolicy)
        self.defaultDecimalsTextEdit.setMinimumSize(QtCore.QSize(0, 25))
        self.defaultDecimalsTextEdit.setMaximumSize(QtCore.QSize(16777215, 25))
        self.defaultDecimalsTextEdit.setObjectName("defaultDecimalsTextEdit")
        self.gridLayout_4.addWidget(self.defaultDecimalsTextEdit, 0, 1, 1, 1)
        self.label_5 = QtWidgets.QLabel(self.groupBox_4)
        self.label_5.setObjectName("label_5")
        self.gridLayout_4.addWidget(self.label_5, 1, 0, 1, 1)
        self.allowedUnitsTextEdit = QtWidgets.QTextEdit(self.groupBox_4)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.allowedUnitsTextEdit.sizePolicy().hasHeightForWidth())
        self.allowedUnitsTextEdit.setSizePolicy(sizePolicy)
        self.allowedUnitsTextEdit.setMinimumSize(QtCore.QSize(0, 25))
        self.allowedUnitsTextEdit.setMaximumSize(QtCore.QSize(16777215, 25))
        self.allowedUnitsTextEdit.setObjectName("allowedUnitsTextEdit")
        self.gridLayout_4.addWidget(self.allowedUnitsTextEdit, 1, 1, 1, 1)
        self.label_6 = QtWidgets.QLabel(self.groupBox_4)
        self.label_6.setObjectName("label_6")
        self.gridLayout_4.addWidget(self.label_6, 2, 0, 1, 1)
        self.preferredUnitsTextEdit = QtWidgets.QTextEdit(self.groupBox_4)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.preferredUnitsTextEdit.sizePolicy().hasHeightForWidth())
        self.preferredUnitsTextEdit.setSizePolicy(sizePolicy)
        self.preferredUnitsTextEdit.setMinimumSize(QtCore.QSize(0, 25))
        self.preferredUnitsTextEdit.setMaximumSize(QtCore.QSize(16777215, 25))
        self.preferredUnitsTextEdit.setObjectName("preferredUnitsTextEdit")
        self.gridLayout_4.addWidget(self.preferredUnitsTextEdit, 2, 1, 1, 1)
        self.horizontalLayout.addLayout(self.gridLayout_4)
        self.gridLayout_3.addWidget(self.groupBox_4, 0, 1, 1, 1)
        self.verticalLayout.addWidget(self.groupBox)
        self.buttonBox = QtWidgets.QDialogButtonBox(Dialog)
        self.buttonBox.setEnabled(True)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.NoButton)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox)

        self.retranslateUi(Dialog)
        self.buttonBox.accepted.connect(Dialog.accept) # type: ignore
        self.buttonBox.rejected.connect(Dialog.reject) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        _translate = QtCore.QCoreApplication.translate
        Dialog.setWindowTitle(_translate("Dialog", "Dialog"))
        self.groupBox_2.setTitle(_translate("Dialog", "Test ScientificSpinBox"))
        self.reprPropertiesGroupBox.setTitle(_translate("Dialog", "representation Properties"))
        self.reprQuantityLabel.setText(_translate("Dialog", "quantity: "))
        self.reprValueLabel.setText(_translate("Dialog", "value:"))
        self.reprUnitLabel.setText(_translate("Dialog", "unit: "))
        self.groupBox_5.setTitle(_translate("Dialog", "base Properties"))
        self.baseValueLabel.setText(_translate("Dialog", "baseValue:"))
        self.baseQuantityLabel.setText(_translate("Dialog", "baseQuantity:"))
        self.baseUnitLabel.setText(_translate("Dialog", "baseUnit:"))
        self.groupBox_6.setTitle(_translate("Dialog", "internal Properties"))
        self.internalTextLabel.setText(_translate("Dialog", "internalText:"))
        self.decimalsLabel.setText(_translate("Dialog", "decimals:"))
        self.lastTextLabel.setText(_translate("Dialog", "lastText:"))
        self.groupBox.setTitle(_translate("Dialog", "Controls"))
        self.groupBox_3.setTitle(_translate("Dialog", "Flags"))
        self.tooltipCheckBox.setText(_translate("Dialog", "Enable tooltip"))
        self.thousandSepCheckBox.setText(_translate("Dialog", "Enable thousand separators"))
        self.integerRoundCheckBox.setText(_translate("Dialog", "Enable rounding to next integer"))
        self.applyButton.setText(_translate("Dialog", "Apply changes"))
        self.groupBox_4.setTitle(_translate("Dialog", "Unit handling"))
        self.label.setText(_translate("Dialog", "Base unit:"))
        self.baseUnitTextEdit.setHtml(_translate("Dialog", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'Sans Serif\'; font-size:9pt; font-weight:400; font-style:normal;\">\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><br /></p></body></html>"))
        self.label_2.setText(_translate("Dialog", "Display unit:"))
        self.displayUnitTextEdit.setHtml(_translate("Dialog", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'Sans Serif\'; font-size:9pt; font-weight:400; font-style:normal;\">\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><br /></p></body></html>"))
        self.label_3.setText(_translate("Dialog", "Default value:"))
        self.defaultValueTextEdit.setHtml(_translate("Dialog", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'Sans Serif\'; font-size:9pt; font-weight:400; font-style:normal;\">\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><br /></p></body></html>"))
        self.label_4.setText(_translate("Dialog", "Default decimals:"))
        self.defaultDecimalsTextEdit.setHtml(_translate("Dialog", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'Sans Serif\'; font-size:9pt; font-weight:400; font-style:normal;\">\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><br /></p></body></html>"))
        self.label_5.setText(_translate("Dialog", "Allowed units:"))
        self.allowedUnitsTextEdit.setHtml(_translate("Dialog", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'Sans Serif\'; font-size:9pt; font-weight:400; font-style:normal;\">\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><br /></p></body></html>"))
        self.label_6.setText(_translate("Dialog", "Preferred units"))
        self.preferredUnitsTextEdit.setHtml(_translate("Dialog", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'Sans Serif\'; font-size:9pt; font-weight:400; font-style:normal;\">\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><br /></p></body></html>"))
