"""
Backend module.

Provides the interfaces between unit manager and ScientificSpinBox
and other resources that shouldn't be coupled with the UI.

Since:
  2024/02/06

Authors:
  - Breno H. Pelegrin S. <breno.pelegrin@usp.br>
"""